@@ -66,6 +66,7 @@ if TYPE_CHECKING:
 from qutebrowser.config import config
 from qutebrowser.misc import objects
 from qutebrowser.utils import qtutils, log, utils, debug, message, version
+from qutebrowser.qt import sip
 
 
 bridge: Optional['NotificationBridgePresenter'] = None
@@ -477,7 +478,9 @@ class SystrayNotificationAdapter(AbstractNotificationAdapter):
     @pyqtSlot(int)
     def on_web_closed(self, notification_id: int) -> None:
         assert notification_id == self.NOTIFICATION_ID, notification_id
-        self._systray.hide()
+        if not sip.isdeleted(self._systray):
+            # This can get called during shutdown
+            self._systray.hide()
 
 
 class MessagesNotificationAdapter(AbstractNotificationAdapter):
