@@ -375,18 +375,6 @@ def is_enum(obj: Any) -> bool:
         return False
 
 
-def pyenum_str(value: enum.Enum) -> str:
-    """Get a string representation of a Python enum value.
-
-    This will have the form of "EnumType.membername", which is the default string
-    representation for Python up to 3.10. Unfortunately, that changes with Python 3.10:
-    https://bugs.python.org/issue40066
-    """
-    if sys.version_info[:2] >= (3, 10):
-        return repr(value)
-    return str(value)
-
-
 def get_repr(obj: Any, constructor: bool = False, **attrs: Any) -> str:
     """Get a suitable __repr__ string for an object.
 
@@ -399,14 +387,8 @@ def get_repr(obj: Any, constructor: bool = False, **attrs: Any) -> str:
     cls = qualname(obj.__class__)
     parts = []
     items = sorted(attrs.items())
-
     for name, val in items:
-        if isinstance(val, enum.Enum):
-            s = pyenum_str(val)
-        else:
-            s = repr(val)
-        parts.append(f'{name}={s}')
-
+        parts.append('{}={!r}'.format(name, val))
     if constructor:
         return '{}({})'.format(cls, ', '.join(parts))
     else:
@@ -687,11 +669,12 @@ def yaml_load(f: Union[str, IO[str]]) -> Any:
             r"of from 'collections\.abc' is deprecated.*"):
         try:
             data = yaml.load(f, Loader=YamlLoader)
-        except ValueError as e:
-            if str(e).startswith('could not convert string to float'):
+        except ValueError as e:  # pragma: no cover
+            pyyaml_error = 'could not convert string to float'
+            if str(e).startswith(pyyaml_error):
                 # WORKAROUND for https://github.com/yaml/pyyaml/issues/168
                 raise yaml.YAMLError(e)
-            raise  # pragma: no cover
+            raise
 
     end = datetime.datetime.now()
 
